#!/usr/bin/python

# Copyright (C) 2013 Canonical
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 3.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import argparse
import imp
import importlib
import os
import sys

from gi.repository import Unity
from gi.repository import GLib

MODULE_TYPE = "python{0}".format(sys.version_info.major)
# An internal type to cover legacy "load_source" imports
SOURCE_TYPE = MODULE_TYPE + "-source"


class PythonScopeLoader(Unity.ScopeLoader):
  def do_get_scopes(self, name, module_type):
    if module_type == MODULE_TYPE:
      # Treat as module name
      module = importlib.import_module(name)
    elif module_type == SOURCE_TYPE:
      modulename, _ = os.path.splitext(os.path.basename(name))
      module = imp.load_source(modulename, name)
    else:
      raise RuntimeError("Unknown module type: {0}".format(module_type))
    return [module.load_scope()]

def load_scope(name, as_module):
  if as_module:
    # Treat as module name
    module = importlib.import_module(name)
  else:
    # Treat as source file name: use the file's base name as a module
    # name in the hope it is unique.
    modulename, _ = os.path.splitext(os.path.basename(name))
    module = imp.load_source(modulename, name)
  return module.load_scope()

def main(argv):
  GLib.threads_init()
  parser = argparse.ArgumentParser(
    description='A host process for Unity scopes written in Python')
  parser.add_argument(
    '-s', dest='as_scope_id', action='store_true',
    help='Treat arguments as scope IDs')
  parser.add_argument(
    '-g', dest='as_group', action='store_true',
    help='Treat arguments as groups')
  parser.add_argument(
    '-m', dest='as_module', action='store_true',
    help='Treat arguments as module names rather than file names')
  parser.add_argument(
    'names', metavar='names', nargs='+', type=str,
    help='The scopes to load')
  options = parser.parse_args(argv[1:])

  if sum([options.as_scope_id, options.as_group, options.as_module]) > 1:
    parser.error('Only one of -s, -g and -m can be used')

  loader = PythonScopeLoader()
  for name in options.names:
    if options.as_scope_id:
      loader.load_scope(name)
    elif options.as_group:
      loader.load_group(name)
    elif options.as_module:
      loader.load_module(name, MODULE_TYPE)
    else:
      loader.load_module(name, SOURCE_TYPE)
  # add manual handling for SIGTERM, since we don't use GLib.MainLoop
  # from python it wouldn't be handled properly
  GLib.unix_signal_add_full(0, 2, lambda x: Unity.ScopeDBusConnector.quit(), None)
  Unity.ScopeDBusConnector.run()

if __name__ == "__main__":
  main (sys.argv)




from gi.overrides import override
from gi.importer import modules
import threading
import sys

Unity = modules['Unity']._introspection_module
from gi.repository import GLib

__all__ = []

class ScopeSearchBase(Unity.ScopeSearchBase):

    def __init__(self):
        Unity.ScopeSearchBase.__init__(self)

    def do_run_async(self, callback, callback_data=None):
        def thread_method():
            try:
                self.run()
            finally:
                callback(self)

        t = threading.Thread(target=thread_method, name="python-search-thread")
        t.start()


class ResultPreviewer(Unity.ResultPreviewer):

    def __init__(self):
        Unity.ResultPreviewer.__init__(self)

    def do_run_async(self, callback, callback_data=None):
        def thread_method():
            preview = None
            try:
                preview = self.run()
            finally:
                callback(self, preview)

        t = threading.Thread(target=thread_method, name="python-preview-thread")
        t.start()


class ResultSet(Unity.ResultSet):

    def __init__ (self):
        Unity.ResultSet.__init__(self)

    def add_result(self, *args, **kwargs):
        if len(args) > 0:
            Unity.ResultSet.add_result(self, *args)
        elif len(kwargs) > 0:
            result = kwargs_to_result_variant(**kwargs)
            Unity.ResultSet.add_result_from_variant(self, result)


def kwargs_to_result_variant(**kwargs):
    uri = None
    icon = ""
    category = 0
    result_type = 0
    mimetype = None
    title = None
    comment = ""
    dnd_uri = None
    metadata = {}

    for col_name, value in kwargs.items():
        if col_name == "uri": uri = value
        elif col_name == "icon": icon = value
        elif col_name == "category": category = value
        elif col_name == "result_type": result_type = value
        elif col_name == "mimetype": mimetype = value
        elif col_name == "title": title = value
        elif col_name == "comment": comment = value
        elif col_name == "dnd_uri": dnd_uri = value
        else:
            if isinstance(value, GLib.Variant):
                metadata[col_name] = value
            elif isinstance(value, str):
                metadata[col_name] = GLib.Variant("s", value)
            elif isinstance(value, int):
                metadata[col_name] = GLib.Variant("i", value)
            elif sys.version_info < (3, 0, 0):
                # unicode is not defined in py3
                if isinstance(value, unicode):
                    metadata[col_name] = GLib.Variant("s", value)

    result = GLib.Variant("(ssuussssa{sv})", (uri, icon, category,
                                              result_type, mimetype,
                                              title, comment, dnd_uri,
                                              metadata))
    return result


def dict_to_variant(metadata_dict):
    metadata = {}

    for name, value in metadata_dict.items():
        if isinstance(value, GLib.Variant):
            metadata[name] = value
        elif isinstance(value, str):
            metadata[name] = GLib.Variant("s", value)
        elif isinstance(value, int):
            metadata[name] = GLib.Variant("i", value)
        elif sys.version_info < (3, 0, 0):
            # unicode is not defined in py3
            if isinstance(value, unicode):
                metadata[name] = GLib.Variant("s", value)

    return GLib.Variant("a{sv}", metadata)


class ScopeResult(Unity.ScopeResult):

    @staticmethod
    def create(*args, **kwargs):
        if len(kwargs) > 0:
            result = kwargs_to_result_variant(**kwargs)
            return Unity.ScopeResult.create_from_variant(result)
        return Unity.ScopeResult.create(*args)


class SearchContext(Unity.SearchContext):

    @staticmethod
    def create(search_query, search_type, filter_state, metadata_dict, result_set, cancellable):
        context = Unity.SearchContext.create(search_query, search_type, filter_state, None, result_set, cancellable)
        if metadata_dict and len(metadata_dict) > 0:
            metadata_variant = dict_to_variant(metadata_dict)
            context.set_search_metadata(Unity.SearchMetadata.create_from_variant(metadata_variant))

        return context

ScopeSearchBase = override(ScopeSearchBase)
__all__.append('ScopeSearchBase')
ResultPreviewer = override(ResultPreviewer)
__all__.append('ResultPreviewer')
ResultSet = override(ResultSet)
__all__.append('ResultSet')
ScopeResult = override(ScopeResult)
__all__.append('ScopeResult')
SearchContext = override(SearchContext)
__all__.append('SearchContext')



#!/usr/bin/python
from gi.repository import Unity, GLib

class TestResultSet(Unity.ResultSet):
    def __init__(self):
        Unity.ResultSet.__init__(self)
        self.results = []

    def do_add_result(self, result):
        assert(result.uri == "file:///foo")
        assert(result.title == "Title")
        assert(len(result.metadata) > 0)
        assert("whatever" in result.metadata)
        # bug in pygi? copy() shouldn't be needed
        self.results.append(result.copy())

rs = TestResultSet()
# overrides are not installed when running tests, so don't use add_result
variant = GLib.Variant('(ssuussssa{sv})', ("file:///foo", "file:///", 0, 0,
                                           "text/plain", "Title", "",
                                           "file:///foo", {'whatever': GLib.Variant("s", "foo")}))
rs.add_result_from_variant(variant)

saved_result = rs.results[0]
assert(saved_result.uri == "file:///foo")
assert(saved_result.title == "Title")
assert(len(saved_result.metadata) > 0)
assert("whatever" in saved_result.metadata)



#! /usr/bin/python

from gi.repository import GObject, GLib, Dee, Unity

class Merger (GObject.Object, Unity.MergeStrategy):
	def __init__ (self):
		GObject.Object.__init__ (self)
	
	def do_merge_result (self, model, row, n_cols):
		print "MERGE", model, row, n_cols
		return model.append_row (row)

m = Merger()
l = Unity.Lens.new ("/test/lens", "testlens")
s = Unity.Scope.new ("/test/scope")
l.props.merge_strategy = m
l.add_local_scope (s)

# For the sake of the example try and add some stuff to the model
# directly. Nroamlly this'd be in response to s search...
s.props.results_model.append ("uri1", "icon", 0, "mimetype", "display-name", "comment", "dnd-uri")
s.props.results_model.append ("uri2", "icon", 0, "mimetype", "display-name", "comment", "dnd-uri")
s.props.results_model.append ("uri3", "icon", 0, "mimetype", "display-name", "comment", "dnd-uri")



  



#!/usr/bin/env python

import os, sys
import subprocess
import re

"""
# This script has two modes of operation:
#
# 1) Capture a trace and generate a html visualization.
#    To use this mode just run `./capture_trace.py`. This will setup a lttng
#    session, enable all userspace trace events and spawns a subshell where
#    you can for example run libunity-tool to perform a search on a scope.
#    When the subshell exits, tracing is stopped and visualization is generated.
#
# 2) Only generate a html visualization of a previously captured trace.
#    Use `./capture_trace.py /path/to/lttng/trace_dir`.
#
# Note that use use the first mode you need to have lttng installed
# and libunity compiled with --enable-lttng option.
# Required packages (besides requirements for libunity's --enable-lttng):
# sudo apt-get install lttng-tools babeltrace
"""

"""
# --- Manually capturing a trace
# lttng create session_name
# lttng enable-event -u -a
# lttng start
# --- start home scope with lttng-enabled branch of libunity, perform a search
# lttng stop
# lttng destroy
# --- view the trace in cli
# babeltrace ~/lttng_traces/<session_name dir>
"""
def capture_lttng_trace():
    session = subprocess.check_output(["lttng", "create", "libunity-trace"], stdin=subprocess.PIPE)
    trace_started = False
    try:
        subprocess.check_output(["lttng", "enable-event", "-u", "-a"], stdin=subprocess.PIPE)
        subprocess.check_output(["lttng", "start"], stdin=subprocess.PIPE)
        trace_started = True
        sys.stderr.write("Trace running... Close this sub-shell with Ctrl+D to finish\n")
        subprocess.check_call(["bash"]) # let this inherit our stdin
    except subprocess.CalledProcessError as cpe:
        if trace_started:
            # bash will return non-zero exit code if you for example Ctrl+C
            # a program within it, let's just ignore those errors
            msg = "Warning: subshell returned error: " + str(cpe.returncode)
            sys.stderr.write(msg + "\n")
        else: raise
    finally:
        if trace_started:
            subprocess.check_output(["lttng", "stop"], stdin=subprocess.PIPE)
        subprocess.check_output(["lttng", "destroy", "libunity-trace"], stdin=subprocess.PIPE)

    match = re.findall(r'Traces will be written in (.+)', session)
    if len(match) > 0: return match[0]
    return None

# trace event example
"""
[20:00:30.697493680] (+0.000014138) miso-ThinkPad:unity-scope-hom:26439 libunity:message: { cpu_id = 0 }, { message = "flush::com.canonical.Unity.Master.Scope.home.T296521243842038" }
"""

def parse_trace_line(line):
    pattern = re.compile(r'^\[(?P<timestamp>[0-9:\.]+)\].+?{ message = "(?P<msg>[^"]+)"')
    match = pattern.match(line)
    timestamp = match.group("timestamp")
    msg = match.group("msg")
    msg_parts = msg.split("::", 2)
    if len(msg_parts) > 1:
        msg_type = msg_parts[0]
        msg = {}
        for kv in msg_parts[1].split(";"):
            items = kv.split("=", 2)
            if len(items) > 1:
                msg[items[0]] = items[1]
            else:
                msg['content'] = items[0]
    else:
        msg_type = "other"
        msg = {'content': msg}

    return (timestamp, msg_type, msg)

def find_end_event(events, searched_event, event_type):
    for i in range(len(events)):
        event = events[i]
        if event[1] == event_type and event[2] == searched_event[2]:
            return i
    return -1

def pair_events(events):
    paired = []
    i = 0
    while i < len(events):
        event = events[i]
        event_type = event[1]
        if event_type.endswith(":start"):
            group = event[1].split(":", 2)[0]
            j = find_end_event(events[i+1:], event, group + ":end")
            if j >= 0:
                end_event = events.pop(i+j+1)
                paired.append((event, end_event))
                i = i+1
                continue
        paired.append((event, None))
        i = i+1

    return paired

COLOR_FOR_EVENT = {
    'search': "'#394a6b'",
    'subsearch': "'#109618'",
    'changeset': "'#990099'",
    'diff': "'#e57357'",
    'flush': "'#ff9900'",
    'push': "'#0099c6'"
}

def produce_html(trace_name, pairs):
    base = """<html><head><title>%s</title></head><body>
<script type="text/javascript" src="https://www.google.com/jsapi?autoload={'modules':[{'name':'visualization',
       'version':'1','packages':['timeline']}]}"></script>
<script type="text/javascript">

google.setOnLoadCallback(drawChart);
function drawChart() {
  var dataTable = new google.visualization.DataTable();

  dataTable.addColumn({ type: 'string', id: 'group' });
  dataTable.addColumn({ type: 'string', id: 'id' });
  dataTable.addColumn({ type: 'string', id: 'data' });
  dataTable.addColumn({ type: 'datetime', id: 'Start' });
  dataTable.addColumn({ type: 'datetime', id: 'End' });

  dataTable.addRows([
    %s
  ]);

  var view = new google.visualization.DataView(dataTable);
  view.setColumns([0,1,3,4]);

  var container = document.getElementById('example2.1');
  var chart = new google.visualization.Timeline(container);

  chart.draw(view);
}
</script>

<div id="example2.1" style="height: 100%%;"></div>
</body></html>"""
    rows = []
    colors = []
    for pair in pairs:
        (start_event, end_event) = pair
        (timestamp, event_type, metadata) = start_event
        has_end_ts = end_event is not None

        group = event_type if not event_type.endswith(":start") else event_type.split(":", 2)[0]
        color = COLOR_FOR_EVENT[group] if group in COLOR_FOR_EVENT else "'#c60000'"
        has_scope_id = 'scope' in metadata
        scope_id = metadata['scope'] if has_scope_id else metadata['content']
        if has_scope_id: del metadata['scope']
        else:
            if group in ['changeset', 'flush', 'diff']:
                match = re.match(r'com.canonical.Unity\.Master\.Scope\.(\w+)\.T', scope_id)
                if match: scope_id = "%s.scope" % match.group(1)
        data = str(metadata).replace("'", "\\'")

        # we're loosing our lovely nanosecond precision :(
        map_to_int = lambda x: int(x[0:3])
        start_date = map(map_to_int, re.findall(r'\d+', timestamp))
        end_date = start_date
        if has_end_ts:
            end_date = map(map_to_int, re.findall(r'\d+', end_event[0]))

        event_name = "%s - %s" % (group, metadata['target']) if 'target' in metadata else group
        data_tuple = (scope_id, group, data, ",".join(map(str,start_date)), ",".join(map(str, end_date)))
        data_format = "[ '%s', '%s', '%s', new Date(0,0,0,%s), new Date(0,0,0,%s) ]"
        data_row = data_format % data_tuple
        rows.append(data_row)
        colors.append(color)
    return base % ("Analysis of %s" % trace_name, ",\n".join(rows))

def get_events_from_babeltrace(trace_dir):
    events = []
    all_events = subprocess.check_output(["babeltrace", trace_dir])
    lines = filter(None, all_events.split("\n"))
    for line in lines:
        event = parse_trace_line(line)
        events.append(event)
    return events

def main(args):
    output_name = None
    if len(args) > 1:
        trace_dir = args[1]
    else:
        trace_dir = capture_lttng_trace()

    if trace_dir:
        basename = os.path.basename(trace_dir)
        if not basename:
            basename = os.path.basename(os.path.dirname(trace_dir))
        output_name = "%s.html" % basename

    events = get_events_from_babeltrace(trace_dir)
    if len(events) == 0:
        raise RuntimeError("There are 0 events in the trace")
    pairs = pair_events(events)
    html = produce_html(trace_dir, pairs)

    if output_name:
        sys.stderr.write("Writing output to '%s'\n" % output_name)
        f = open(output_name, 'w')
        f.write(html)
        f.close()
    else:
        print html

if __name__ == "__main__":
    main (sys.argv)




#!/usr/bin/python
from gi.repository import Unity, Gio

category_set = Unity.CategorySet.new()
cat = Unity.Category.new("example", "Example", Gio.ThemedIcon.new("test"), Unity.CategoryRenderer.DEFAULT)
category_set.add(cat)
cat = Unity.Category.new("another", "Another", Gio.ThemedIcon.new("test"), Unity.CategoryRenderer.GRID)
category_set.add(cat)
cat_list = category_set.get_categories()
#if the binding is broken there'll be double free
del cat_list
del category_set



from gi.repository import Unity, Gio, GObject, Dbusmenu

loop = GObject.MainLoop()

# Pretend to be evolution for the sake of the example 
launcher = Unity.LauncherEntry.get_for_desktop_id ("firefox.desktop")

# Show a count of 124 on the icon
launcher.set_property("count", 124)
launcher.set_property("count_visible", True)

# Set progress to 42% done 
launcher.set_property("progress", 0.42)
launcher.set_property("progress_visible", True)

# Set us as urgent. Quickly! Go! Go! Go! Now! Now!
launcher.set_property("urgent", True)

# We also want a quicklist 
ql = Dbusmenu.Menuitem.new ()
item1 = Dbusmenu.Menuitem.new ()
item1.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Item 1")
item1.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
item2 = Dbusmenu.Menuitem.new ()
item2.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Item 2")
item2.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
ql.child_append (item1)
ql.child_append (item2)
launcher.set_property("quicklist", ql)

loop.run()
